﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using HeuristicLab.Collections;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Analysis.Views {
  [View("Histogram View")]
  [Content(typeof(Histogram), IsDefaultView = true)]
  public sealed partial class HistogramView : NamedItemView {

    public new Histogram Content {
      get { return (Histogram)base.Content; }
      set { base.Content = value; }
    }

    public HistogramView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.Values.ItemsAdded -= new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsAdded);
      Content.Values.ItemsRemoved -= new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsRemoved);
      Content.Values.CollectionReset -= new CollectionItemsChangedEventHandler<double>(Content_Values_CollectionReset);
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.Values.ItemsAdded += new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsAdded);
      Content.Values.ItemsRemoved += new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsRemoved);
      Content.Values.CollectionReset += new CollectionItemsChangedEventHandler<double>(Content_Values_CollectionReset);
    }

    #region Event Handlers (Content)
    private void Content_Values_ItemsAdded(object sender, CollectionItemsChangedEventArgs<double> e) {
      UpdateChartSettings();
    }
    private void Content_Values_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<double> e) {
      InitializeChart();
    }
    private void Content_Values_CollectionReset(object sender, CollectionItemsChangedEventArgs<double> e) {
      InitializeChart();
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        enhancedChart.Series.Clear();
      } else {
        InitializeChart();
      }
    }

    private void InitializeChart() {
      enhancedChart.Series.Clear();
      UpdateChartSettings();
    }

    private void UpdateChartSettings() {
      if (Content == null || Content.Values.Count < 1) return;

      int bins = (int)numericUpDown.Value;
      string histogramSeriesName = "Histogram";

      Series histogramSeries = null;
      if (enhancedChart.Series.IndexOf(histogramSeriesName) < 0) {
        histogramSeries = enhancedChart.Series.Add(histogramSeriesName);

        histogramSeries.ChartType = SeriesChartType.Column;
        histogramSeries.BorderColor = Color.Black;
        histogramSeries.BorderWidth = 1;
        histogramSeries.BorderDashStyle = ChartDashStyle.Solid;
      } else {
        histogramSeries = enhancedChart.Series[histogramSeriesName];
        histogramSeries.Points.Clear();
      }

      double minValue = Content.Values.Min();
      double maxValue = Content.Values.Max();
      double intervalWidth = (maxValue - minValue) / bins;

      double current = minValue;
      int count = 0;
      foreach (double v in Content.Values.OrderBy(x => x)) {
        if (v < current + intervalWidth) {
          count++;
        } else {
          histogramSeries.Points.AddXY(current + intervalWidth / 2.0, count);
          count = 0;
          current += intervalWidth;
        }
      }

      histogramSeries["PointWidth"] = "1";

      ChartArea chartArea = enhancedChart.ChartAreas[histogramSeries.ChartArea];
      chartArea.AxisY.Title = "Frequency";
      chartArea.AxisX.Minimum = minValue;
      chartArea.AxisX.Maximum = maxValue;

      double axisInterval = intervalWidth;
      while ((maxValue - minValue) / axisInterval > 10.0) {
        axisInterval *= 2.0;
      }
      chartArea.AxisX.Interval = axisInterval;
    }


    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      enhancedChart.Enabled = Content != null;
    }

    #region Event Handlers (child controls)
    private void numericUpDown_ValueChanged(object sender, System.EventArgs e) {
      UpdateChartSettings();
    }
    #endregion
  }
}
